@@ -7,9 +7,15 @@ import multiprocessing
 import random
 import subprocess
 import errno
+import traceback
+import socket
+import signal
+
 try:
     import ConfigParser as configparser
+    from urllib2 import URLError
 except ImportError:
+    from urllib.error import URLError
     import configparser as configparser
 
 from PyQt4.QtCore import *
@@ -21,6 +27,9 @@ import pytrailer as amt
 from .qtcustom import *
 from .qtcustom import resources
 from .downloader import TrailerDownloader, DownloadStatus
+from logger import log
+
+term_closing = 0
 
 
 class PyTrailerWidget(QMainWindow):
@@ -31,6 +40,7 @@ class PyTrailerWidget(QMainWindow):
 
     def __init__(self, *args):
         QMainWindow.__init__(self, *args)
+        log.debug("main window initialization starting")
         # these are all categories apple is providing for now
         self.categories = [(self.tr('Just added'), '/trailers/home/feeds/just_added.json'),
                      (self.tr('Exclusive'), '/trailers/home/feeds/exclusive.json'),
@@ -45,7 +55,9 @@ class PyTrailerWidget(QMainWindow):
                                        'parallelDownload':'2',
                                        'player':'mplayer -user-agent %%a %%u'})
 
+        log.info("settings loaded: %s" % self.config.items("DEFAULT"))
         # run initializations
+        self.player_proc = None
         self.list_loader = None
         self.list_loader_p = None
         self.movieDict = {}
@@ -55,12 +67,15 @@ class PyTrailerWidget(QMainWindow):
         self.init_widget()
         self.init_menus()
         self.downloader.start()
+        signal.signal(signal.SIGTERM, PyTrailerWidget.term_handler)
+        signal.signal(signal.SIGINT, PyTrailerWidget.term_handler)
+        log.debug("main window initialization done")
 
     def init_widget(self):
         """Initialize main child widgets, layouts etc."""
 
         self.refreshTimer = QTimer(self)
-        self.refreshTimer.timeout.connect(self.refresh_movies)
+        self.refreshTimer.timeout.connect(self.refresh_wrapper)
         self.refreshTimer.start(1000)
         self.setWindowTitle(self.tr("PyTrailer - Apple Trailer Downloader"))
 
@@ -194,6 +209,7 @@ class PyTrailerWidget(QMainWindow):
         """Current trailer groups gets unloaded and preloader
         processes stopped
         """
+        log.debug("unloading previous group")
         while not self.readAheadTaskQueue.empty():
             self.readAheadTaskQueue.get()
 
@@ -227,6 +243,7 @@ class PyTrailerWidget(QMainWindow):
                 return
 
         self.unload_current_group()
+        log.debug("loading group %s" % groupName)
         # loadID is used to identify what group task belonged to
         # we can use it to make sure we don't display trailers from
         # different group after being cached
@@ -241,6 +258,7 @@ class PyTrailerWidget(QMainWindow):
 
 
     def save_config(self):
+        log.debug("saving config file")
         with open(self.configPath, 'w') as configfile:
             self.config.write(configfile)
 
@@ -249,14 +267,29 @@ class PyTrailerWidget(QMainWindow):
         for p in self.readAheadProcess:
             p.terminate()
         self.list_loader_p.terminate()
+        if self.player_proc:
+            self.player_proc.poll()
+            if self.player_proc.returncode is None:
+                self.player_proc.terminate()
         self.save_config()
         self.save_cache()
+        log.debug("closing application")
+
+    def refresh_wrapper(self):
+        if not term_closing:
+            self.refresh_movies()
+        else:
+            self.refreshTimer.stop()
+            self.close()
 
     def refresh_movies(self):
         # if we are loading new group and movie list is ready, get it
         if self.list_loader and self.list_loader.poll():
-            self.movieList = self.list_loader.recv()
-            self.display_group()
+            exception, self.movieList = self.list_loader.recv()
+            if exception:
+                self.report_network_problem(exception)
+            else:
+                self.display_group()
             self.list_loader_p.join()
         # let's refresh status of trailer downloads
         self.refresh_download_status()
@@ -348,9 +381,26 @@ class PyTrailerWidget(QMainWindow):
         if len(list(self.trailerDownloadDict.keys())) and not self.statusView.isVisible():
             self.statusView.setVisible(True)
 
+        if self.player_proc:
+            self.player_proc.poll()
+            ret = self.player_proc.returncode
+            if ret and ret != 0:
+                log.error("player return code was non-null. Player output:")
+                log.error(self.player_proc.stderr.read())
+                QMessageBox.critical(self,
+                                self.tr("Player error"),
+                                self.tr(
+"""Player indicates error while playing selected trailer.
+Please verify player configuration is correct.
+For player error output see log file
+"""))
+                self.player_proc = None
+
+
     def download_trailer(self, url):
         """Adds appropriate tasks to download trailer and sets initial
         DownloadStatus"""
+        log.info("initializing download of %s" % url)
         self.trailerDownloadQueue.put((str(url),
                                       self.config.get("DEFAULT","downloadDir")))
         self.trailerDownloadDict[str(url)] = DownloadStatus(str(url),
@@ -364,10 +414,28 @@ class PyTrailerWidget(QMainWindow):
             elif player[i] == '%u':
                 player[i] = url
 
-        subprocess.Popen(player)
+        log.info("running player: %s" % player)
+        try:
+            self.player_proc = subprocess.Popen(player, stderr=subprocess.PIPE)
+        except OSError as e:
+            log.error("player could not be executed")
+            log.error(traceback.format_exc())
+            if e.errno == errno.ENOENT:
+                # player doesn't exist
+                QMessageBox.critical(self,
+                                self.tr("Player executable not found"),
+                                self.tr("Player could not be run.\nPlease verify player configuration is correct"))
+            else:
+                QMessageBox.critical(self,
+                                self.tr("Player execution failure"),
+                                self.tr("Unknown error while running player."))
+
+
+
 
     def add_to_cache(self, movie):
         """Adds movie to disk cache"""
+        log.debug("adding movie to cache: %s" % movie.title)
         latestUpdate = movie.get_latest_trailer_date()
         self.movie_cache[movie.baseURL] = (latestUpdate,
                                            movie.poster,
@@ -376,6 +444,7 @@ class PyTrailerWidget(QMainWindow):
 
     def load_cache(self):
         """Loads trailer info cache from disk"""
+        log.debug("loading cache from disk")
         try:
             with open(self.cachePath,"rb") as f:
                 self.movie_cache = pickle.load(f)
@@ -388,9 +457,18 @@ class PyTrailerWidget(QMainWindow):
 
     def save_cache(self):
         """Saves trailer info cache to disk"""
+        log.debug("saving cache to disk")
         with open(self.cachePath,"wb") as f:
             pickle.dump(self.movie_cache, f)
 
+    def report_network_problem(self, exception):
+        QMessageBox.critical(self,
+                             self.tr("Network error"),
+                             self.tr(
+"""There was a network problem when downloading movie list.
+Please check your network settings.
+"""))
+
 
     @staticmethod
     def movie_readahead(taskQueue, doneQueue, cache):
@@ -398,8 +476,9 @@ class PyTrailerWidget(QMainWindow):
         caching additional movie information
         """
         while True:
-            i, movie, loadID = taskQueue.get()
             try:
+                i, movie, loadID = taskQueue.get()
+                log.debug("loading information about movie %s" % movie.title)
                 latestUpdate = movie.get_latest_trailer_date()
                 if movie.baseURL in cache and cache[movie.baseURL][0] >= latestUpdate:
                     cached_data = cache[movie.baseURL]
@@ -411,7 +490,15 @@ class PyTrailerWidget(QMainWindow):
                     movie.trailerLinks
                     movie.description
                 doneQueue.put((i, movie, loadID))
+            except socket.error as e:
+                log.error("network problem error while loading movie information: %s" % e)
+                log.error(traceback.format_exc())
+            except KeyboardInterrupt:
+                log.debug("keyboard interrupt. stopping movie readahead")
+                return
             except:
+                log.error("uncaught exception ocurred while doing readahead. Please report this!")
+                log.error(traceback.format_exc())
                 raise
 
     @staticmethod
@@ -419,13 +506,27 @@ class PyTrailerWidget(QMainWindow):
         """Function to be run in a separate process. It will send
         movie list through pipe. Use poll() to verify that data is ready.
         """
-        conn.send(amt.getMoviesFromJSON(url))
-        conn.close()
+        try:
+            log.info("getting movie list from %s" % url)
+            movies = amt.getMoviesFromJSON(url)
+            conn.send((None, movies))
+            conn.close()
+        except URLError as e:
+            log.error("network problem error while loading movie list: %s" % e)
+            log.error(traceback.format_exc())
+            conn.send((e, None))
+        except socket.error as e:
+            log.error("network problem error while loading movie list: %s" % e)
+            log.error(traceback.format_exc())
+            conn.send((e, None))
+        except Exception as e:
+            log.error("uncaught exception ocurred while loading movie list. Please report this!")
+            log.error(traceback.format_exc())
+            conn.send((e, None))
 
 
-if __name__ == "__main__":
-    app = QApplication(sys.argv)
-    widget = PyTrailerWidget()
-    widget.resize(800, 600)
-    widget.show()
-    sys.exit(app.exec_())
+    @staticmethod
+    def term_handler(signum, frame):
+        """Handles closing of main process."""
+        global term_closing
+        term_closing = 1
